function rec=LBregDebP(g,ker,blur,sigma,mu,delta,theta,ForC,frame,Level,wLevel)

%rec=LBregDebP(g,ker,blur,sigma,mu,delta,theta,ForC,frame,Level,wLevel)
%Linearized Bregman iteration for Deblurring with Preconditioning.
%The tight frame systems are framlets.
%Input:   g -- blurred image;
%       ker -- blurring kernel;
%      blur -- function handle of blurring of usage blur(img,ker).
%     sigma -- the sigma of noise
% mu, delta -- mu and delta in linearized Bregman iteration.
%     theta -- regularized parameter theta used in preconditioner.
%     ForC -- 'F' FFT-based preconditioner; 'C' DCT-based preconditioner. 
%     frame -- framelet system used. 0-Haar, 1-Piecewise Linear, 3-Picewise Cubic
%     Level -- framelet decomposition level.
%    wLevel -- weighting of thresholding on each level. If wLevel<=0, we 
%              use default values, the product of norms of filters. If 
%              wLevel>0, the thresholding is multiplied by wLevel from the 
%              current level to the next level.

[D,R]=GenerateFrameletFilter(frame);
nD=length(D);
[m,n]=size(g);
cker=fliplr(flipud(ker));

if ForC=='C'    
    Transform=@(x)dct2(x);
    InverseTransform=@(x)idct2(x);
elseif ForC=='F'
    Transform=@(x)fft2(x);
    InverseTransform=@(x)ifft2(x); 
else
    disp(['ForC must be F or C.']);
    rec=0;
    return;
end

%Compute the eigenvalues of P.
eigenP=eigenofP(ker,cker,ForC,theta,m,n);  

%Compute the weighted thresholding parameters.
muLevel=getwThresh(mu,wLevel,Level,D);

%initialization
u=FraDecMultiLevel(zeros(m,n),D,Level);
v=u;
r=g;
nres=sqrt(sum(g(:).^2));
normf=nres;
step=0;
stdr=std(r(:));

%Start Iteration
while (stdr>sigma)
    dr=Transform(r);dr=dr./eigenP;zr=InverseTransform(dr);
    zr=blur(zr,cker);
    temp=FraDecMultiLevel(zr,D,Level);
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                v{ki}{ji,jj}=v{ki}{ji,jj}+temp{ki}{ji,jj};
                if (ji==1)&(jj==1)
                    u{Level}{1,1}=delta*v{Level}{1,1};
                else
                    u{ki}{ji,jj}=wthresh(delta*v{ki}{ji,jj},'s',delta*muLevel{ki}{ji,jj});
                end
            end
        end
    end
    rec=FraRecMultiLevel(u,R,Level);
    r=g-blur(rec,ker);
    nres=sqrt(sum(r(:).^2));
    stdr=std(r(:));
    step=step+1;
    disp(['error on step ' num2str(step) ' is ' num2str(nres/normf) '; and the std of the error is ' num2str(stdr)]);
end

rec(rec>255)=255;rec(rec<0)=0;

function [D,R]=GenerateFrameletFilter(frame)
if frame==0          %Haar Wavelet
    D{1}=[0 1 1]/2;
    D{2}=[0 1 -1]/2;
    D{3}='cc';
    R{1}=[1 1 0]/2;
    R{2}=[-1 1 0]/2;
    R{3}='cc';
elseif frame==1      %Piecewise Linear Framelet
    D{1}=[1 2 1]/4;
    D{2}=[1 0 -1]/4*sqrt(2);
    D{3}=[-1 2 -1]/4;
    D{4}='sss';
    R{1}=[1 2 1]/4;
    R{2}=[-1 0 1]/4*sqrt(2);
    R{3}=[-1 2 -1]/4;
    R{4}='sas';
elseif frame==3      %Piecewise Cubic Framelet
    D{1}=[1 4 6 4 1]/16;
    D{2}=[1 2 0 -2 -1]/8;
    D{3}=[-1 0 2 0 -1]/16*sqrt(6);
    D{4}=[-1 2 0 -2 1]/8;
    D{5}=[1 -4 6 -4 1]/16;
    D{6}='sssss';
    R{1}=[1 4 6 4 1]/16;
    R{2}=[-1 -2 0 2 1]/8;
    R{3}=[-1 0 2 0 -1]/16*sqrt(6);
    R{4}=[1 -2 0 2 -1]/8;
    R{5}=[1 -4 6 -4 1]/16;
    R{6}='sasas';
end

function eigenP=eigenofP(ker,cker,ForC,theta,m,n);  
if ForC=='C'
    tmp=zeros(m,n);tmp(1,1)=1;dtmp=dct2(tmp);
    tmpker=conv2(ker,cker,'full');
    [km,kn]=size(tmpker);fkm=floor(km/2);fkn=floor(kn/2);
    tmp=tmpker(fkm+1:km,fkn+1:kn);
    tmp(1:fkm,1:fkn)=tmp(1:fkm,1:fkn)+fliplr(flipud(tmpker(1:fkm,1:fkn)));
    tmp(1:fkm,1:fkn+1)=tmp(1:fkm,1:fkn+1)+flipud(tmpker(1:fkm,fkn+1:kn));
    tmp(1:fkm+1,1:fkn)=tmp(1:fkm+1,1:fkn)+fliplr(tmpker(fkm+1:km,1:fkn));
    ctmp=zeros(m,n);ctmp(1:fkm+1,1:fkn+1)=tmp;
    ctmp=dct2(ctmp);
    eigen=ctmp./dtmp;

    tmp=[1;-1;zeros(m-2,1)];
    tmp=dct(tmp);eigenG=(tmp./dct([1;zeros(m-1,1)]))*ones(1,n);
    tmp=[1 -1 zeros(1,n-2)];
    tmp=dct(tmp);
    eigenG=eigenG+ones(m,1)*(tmp./dct([1 zeros(1,n-1)]));
    
    eigenP=eigen+theta*eigenG;
elseif ForC=='F'
    [nker,mker]=size(ker);
    tmp=zeros(m,n);tmp(1:nker,1:mker)=ker;
    tmp=circshift(tmp,[-floor(nker/2),-floor(mker/2)]);
    eigen=abs(fft2(tmp)).^2;

    tmp=[1;zeros(m-2,1);-1];
    tmp=fft(tmp);eigenG=abs(tmp*ones(1,n)).^2;
    tmp=[1 zeros(1,n-2) -1];
    tmp=fft(tmp);
    eigenG=eigenG+abs(ones(m,1)*tmp).^2;
    
    eigenP=eigen+theta*eigenG;
else
    disp(['wrong']);
    return;
end

function muLevel=getwThresh(mu,wLevel,Level,D);
nfilter=1;
nD=length(D);
if wLevel<=0
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                muLevel{ki}{ji,jj}=mu*nfilter*norm(D{ji})*norm(D{jj});
            end
        end
        nfilter=nfilter*norm(D{1});
    end
else
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                muLevel{ki}{ji,jj}=mu*nfilter;
            end
        end
        nfilter=nfilter*wLevel;
    end
end